@@ -1,4 +1,8 @@
-#' visualise
+#' Visualise quality-control metrics using a shiny app
+#'
+#' This function allows to visualise several quality-control metrics
+#' in an interactive [shiny] application, in which thresholds can be customised
+#' and exported to the global environment.
 #'
 #' @inheritParams normalise
 #'
@@ -80,9 +84,6 @@ visualise <- function(nacho_object) {
       '  "summarise()" and/or "normalise()" must be called before "visualise()".'
     )
   }
-  if (!interactive()) {
-    stop('[NACHO] Must be run in an interactive R session!')
-  }
 
   # nocov start
   id_colname <- nacho_object[["access"]]
@@ -160,7 +161,7 @@ visualise <- function(nacho_object) {
     ),
     server = function(input, output) {
       shiny::observeEvent(input$do, {
-        file_ext <- function (x) {
+        file_ext <- function(x) {
             pos <- regexpr("\\.([[:alnum:]]+)$", x)
             ifelse(pos > -1L, substring(x, pos + 1L), "")
         }
@@ -248,35 +249,101 @@ visualise <- function(nacho_object) {
           "about" = {
             vignette_exists <- system.file("doc", "NACHO.Rmd", package = "NACHO")
             if (nchar(vignette_exists)==0) {
-              p("The vignette was not build.")
+              shiny::p("The vignette was not build.")
             }
-
-            p(
-              shiny::br(),
-              shiny::em("NACHO"), " (NanoString Quality Control Dashboard) is developed for NanoString nCounter data.",
-              shiny::br(),
-              "NanoString nCounter data is a mRNA or miRNA expression assay and works with fluorescent barcodes.",
-              shiny::br(),
-              "Each barcode is assigned a mRNA/miRNA, which can be counted after bonding with its target.",
-              shiny::br(),
-              "As a result each count of a specific barcode represents the presence of its target mRNA/miRNA.",
-              shiny::br(),
-              shiny::br(),
-              shiny::em("NACHO"), ' is able to load, visualise and normalise the exported NanoString nCounter data and facilitates the user in performing a quality control.',
-              shiny::br(),
-              shiny::em("NACHO"), ' does this by visualising Quality Control metrics, expression of control genes, principal components and sample specific size factors in an interactive web application.',
-              shiny::br(),
-              'With the use of two functions, RCC files are summarised and visualised, namely: ', shiny::code("summarise()"), ' and ', shiny::code("visualise()"), '.',
-              shiny::br(),
-              'The ', shiny::code("summarise()"), ' function is used to preprocess the data.',
-              shiny::br(),
-              'The ', shiny::code("visualise()"), ' function initiates a RStudio Shiny-based dashboard that visualises all relevant QC plots.',
-              shiny::br(),
-              shiny::br(),
-              shiny::em("NACHO"), ' also includes a function ', shiny::code("normalise()"), ', which calculates sample specific size factors and normalises the data.',
-              shiny::br(),
-              shiny::code("normalise()"), ' creates a list in which your settings, the raw counts and normalised counts are stored.'
+            details <- c(
+              "NACHO" = paste(
+                '*NACHO* (**NA**nostring quality **C**ontrol das**H**b**O**ard) is developed for NanoString nCounter data.  ',
+                'NanoString nCounter data is a messenger-RNA/micro-RNA (mRNA/miRNA) expression assay and works with fluorescent barcodes.  ',
+                'Each barcode is assigned a mRNA/miRNA, which can be counted after bonding with its target.  ',
+                'As a result each count of a specific barcode represents the presence of its target mRNA/miRNA.',
+                '\n',
+                '*NACHO* is able to load, visualise and normalise the exported NanoString nCounter data and facilitates the user in performing a quality control.  ',
+                '*NACHO* does this by visualising quality control metrics, expression of control genes, principal components and sample specific size factors in an interactive web application.',
+                '\n',
+                'With the use of two functions, RCC files are summarised and visualised, namely: `summarise()` and `visualise()`.',
+                '\n',
+                '* The `summarise()` function is used to preprocess the data.',
+                '* The `visualise()` function initiates a [Shiny-based dashboard](https://shiny.rstudio.com/) that visualises all relevant QC plots.',
+                '\n',
+                '*NACHO* also includes a function `normalise()`, which (re)calculates sample specific size factors and normalises the data.',
+                '\n',
+                '* The `normalise()` function creates a list in which your settings, the raw counts and normalised counts are stored.',
+                '\n',
+                'In addition (since v0.6.0) *NACHO* includes two (three) additional functions:',
+                '\n',
+                '* The `render()` function renders a full quality-control report (HTML) based on the results of a call to `summarise()` or `normalise()` (using `print()` in a Rmarkdown chunk).',
+                '* The `autoplot()` function draws any quality-control metrics from `visualise()` and `render()`.',
+                '\n',
+                'For more `vignette("NACHO")`.',
+                sep = "\n"
+              ),
+              "Binding Density" = paste(
+                "The imaging unit only counts the codes that are unambiguously distinguishable.",
+                "It simply will not count codes that overlap within an image.",
+                "This provides increased confidence that the molecular counts you receive are from truly recognisable codes.",
+                "Under most conditions, forgoing the few barcodes that do overlap will not impact your data.",
+                "Too many overlapping codes in the image, however, will create a condition called image saturation in which significant data loss could occur (critical data loss from saturation is uncommon).",
+                "\n",
+                "To determine the level of image saturation, the nCounter instrument calculates the number of optical features per square micron for each lane as it processes the images.",
+                "This is called the **Binding Density**.",
+                "The **Binding Density** is useful for determining whether data collection has been compromised due to image saturation.",
+                "The acceptable range for **Binding Density** is:",
+                "\n",
+                "* `0.1 - 2.25` for **MAX**/**FLEX** instruments",
+                "* `0.1 - 1.8` for **SPRINT** instruments",
+                "\n",
+                "Within these ranges, relatively few reporters on the slide surface will overlap, enabling the instrument to accurately tabulate counts for each reporter species.",
+                "A **Binding Density** significantly greater than the upper limit in either range is indicative of overlapping reporters on the slide surface.",
+                "The counts observed in lanes with a **Binding Density** at this level may have had significant numbers of codes ignored, which could potentially affect quantification and linearity of the assay.",
+                "Some of the factors that may contribute to increased **Binding Density** are listed in the Factors affecting **Binding Density** box.",
+                sep = "\n"
+              ),
+              "Field of View (Imaging)" = paste(
+                "Each individual lane scanned on an nCounter system is divided into a few hundred imaging sections, called Fields of View (**FOV**), the exact number of which will depend on the system being used (*i.e.*, **MAX/FLEX** or **SPRINT**), and the scanner settings selected by the user.",
+                "The system images these FOVs separately, and sums the barcode counts of all **FOV**s from a single lane to form the final raw data count for each unique barcode target.",
+                "Finally, the system reports the number of **FOV**s successfully imaged as FOV Counted.",
+                "\n",
+                "Significant discrepancy between the number of **FOV** for which imaging was attempted (**FOV Count**) and for which imaging was successful (**FOV Counted**) may indicate an issue with imaging performance.",
+                "Recommended percentage of registered FOVs (*i.e.*, **FOV Counted** over **FOV Count**) is `75 %`.",
+                "Lanes will be flagged if this percentage is lower.",
+                sep = "\n"
+              ),
+              "Positive Control linearity" = paste(
+                "Six synthetic DNA control targets are included with every nCounter Gene Expression assay.",
+                "Their concentrations range linearly from `128 fM` to `0.125 fM`, and they are referred to as **POS_A** to **POS_F**, respectively.",
+                "These **Positive Controls** are typically used to measure the efficiency of the hybridization reaction, and their step-wise concentrations also make them useful in checking the linearity performance of the assay.",
+                "An R2 value is calculated from the regression between the known concentration of each of the **Positive Controls** and the resulting counts from them (this calculation is performed using log2-transformed values).",
+                "\n",
+                "Since the known concentrations of the **Positive Controls** increase in a linear fashion, the resulting counts should, as well.",
+                "Therefore, R2 values should be higher than `0.95`.",
+                "\n",
+                "Note that because POS_F has a known concentration of `0.125 fM`, which is considered below the limit of detection of the system, it should be excluded from this calculation (although you will see that **POS_F** counts are significantly higher than the negative control counts in most cases).",
+                sep = "\n"
+              ),
+              "Limit of Detection" = paste(
+                "The limit of detection is determined by measuring the ability to detect **POS_E**, the `0.5 fM` positive control probe, which corresponds to about 10,000 copies of this target within each sample tube.",
+                "On a **FLEX**/**MAX** system, the standard input of `100 ng` of total RNA will roughly correspond to about 10,000 cell equivalents (assuming one cell contains `10 pg` total RNA on average).",
+                "An nCounter assay run on the **FLEX**/**MAX** system should thus conservatively be able to detect roughly one transcript copy per cell for each target (or 10,000 total transcript copies).",
+                "In most assays, you will observe that even the **POS_F** probe (equivalent to 0.25 copies per cell) is detectable above background.",
+                "\n",
+                "To determine whether **POS_E** is detectable, it can be compared to the counts for the negative control probes.",
+                "Every nCounter Gene Expression assay is manufactured with eight negative control probes that should not hybridize to any targets within the sample.",
+                "Counts from these will approximate general non-specific binding of probes within the samples being run.",
+                "The counts of **POS_E** should be higher than two times the standard deviation above the mean of the negative control.",
+                sep = "\n"
+              )
             )
+            about_md <- tempfile(fileext = "_about.md")
+            unlink(about_md)
+            for (iname in names(details)) {
+              cat(
+                paste0("\n\n### ", iname, "\n\n", details[iname], "\n"),
+                file = about_md,
+                append = TRUE
+              )
+            }
+            shiny::p(shiny::includeMarkdown(about_md))
           }
         )
       })
@@ -296,7 +363,7 @@ visualise <- function(nacho_object) {
               "StagePosition"
             )
           )
-        } else if (input$maintabs == "vis" | input$maintabs == "norm" | input$maintabs == "cg") {
+        } else if (input$maintabs %in% c("vis", "norm", "cg")) {
           shiny::radioButtons(
             inputId = "colour_choice",
             label = "Select colour attributes:",
@@ -319,7 +386,7 @@ visualise <- function(nacho_object) {
             choices = colour_variables,
             selected = "CodeClass"
           )
-        } else if (input$maintabs == "vis" | input$maintabs == "norm" | input$maintabs == "cg") {
+        } else if (input$maintabs %in% c("vis", "norm", "cg")) {
            colour <- ifelse(is.null(input$colour_choice), TRUE, input$colour_choice)
           if (colour) {
             shiny::selectInput(
@@ -345,45 +412,53 @@ visualise <- function(nacho_object) {
       })
 
       output$interfaceC <- shiny::renderUI({
-        shiny::req(input$maintabs%in%c("met", "vis"))
-        if (input$maintabs == "met") {
-          shiny::div(
-            shiny::checkboxInput(
-              inputId = "outlier",
-              label = "View outliers",
-              value = TRUE
-            ),
-            align = "center"
-          )
-        } else if (input$maintabs == "vis") {
-          shiny::req(input$tabs == "prin")
-          shiny::selectInput(
-            inputId = "pcA_sel",
-            label = "PC on x-axis:",
-            choices = grep("^PC[0-9]+$", colnames(nacho), value = TRUE)
-          )
-        }
+        shiny::req(input$maintabs %in% c("met", "vis"))
+        switch(
+          EXPR = input$maintabs,
+         "met" = {
+            shiny::div(
+              shiny::checkboxInput(
+                inputId = "outlier",
+                label = "View outliers",
+                value = TRUE
+              ),
+              align = "center"
+            )
+          },
+          "vis" = {
+            shiny::req(input$tabs == "prin")
+            shiny::selectInput(
+              inputId = "pcA_sel",
+              label = "PC on x-axis:",
+              choices = grep("^PC[0-9]+$", colnames(nacho), value = TRUE)
+            )
+          }
+        )
       })
 
       output$interfaceD <- shiny::renderUI({
-        shiny::req(input$maintabs%in%c("met", "vis"))
-        if (input$maintabs == "met") {
-          shiny::div(
-            shiny::checkboxInput(
-              inputId = "outlab",
-              label = "View outlier labels"
-            ),
-            align = "center"
-          )
-        } else if (input$maintabs == "vis") {
-          shiny::req(input$tabs == "prin")
-          shiny::selectInput(
-            inputId = "pcB_sel",
-            label = "PC on y-axis:",
-            choices = grep("^PC[0-9]+$", colnames(nacho), value = TRUE),
-            selected = grep("^PC[0-9]+$", colnames(nacho), value = TRUE)[2]
-          )
-        }
+        shiny::req(input$maintabs %in% c("met", "vis"))
+        switch(
+          EXPR = input$maintabs,
+         "met" = {
+            shiny::div(
+              shiny::checkboxInput(
+                inputId = "outlab",
+                label = "View outlier labels"
+              ),
+              align = "center"
+            )
+          },
+          "vis" = {
+            shiny::req(input$tabs == "prin")
+            shiny::selectInput(
+              inputId = "pcB_sel",
+              label = "PC on y-axis:",
+              choices = grep("^PC[0-9]+$", colnames(nacho), value = TRUE),
+              selected = grep("^PC[0-9]+$", colnames(nacho), value = TRUE)[2]
+            )
+          }
+        )
       })
 
       output$interfaceE1 <- shiny::renderUI({
@@ -441,7 +516,7 @@ visualise <- function(nacho_object) {
               as.numeric(ranges["BD4"])
             )
           )
-        } else if (input$tabs == "FoV" | input$tabs == "LoD" | input$tabs == "PC") {
+        } else if (input$tabs %in% c("FoV", "LoD", "PC")) {
           shiny::sliderInput(
             inputId = "threshold",
             label = sprintf("Custom QC threshold (default: %s)", as.numeric(ranges[paste0(input$tabs, "3")])),
@@ -473,7 +548,7 @@ visualise <- function(nacho_object) {
       })
 
       output$outlier_table <- shiny::renderDataTable({
-        shiny::req(input$maintabs%in%c("ot"))
+        shiny::req(input$maintabs == "ot")
         details_out <- details_outlier(
           nacho_df = nacho,
           id_colname = id_colname,
@@ -493,7 +568,7 @@ visualise <- function(nacho_object) {
       })
 
       output$all <- shiny::renderPlot({
-        shiny::req(!input$maintabs%in%c("ot", "about"))
+        shiny::req(!input$maintabs %in% c("ot", "about"))
         # Prepare axis text
         labels <- c(
           "MC" = "Average Counts",
@@ -528,7 +603,7 @@ visualise <- function(nacho_object) {
             if (input$tabs == "BD") {
               local_data <- nacho[nacho[[input$tabs]] >= input$threshold[1] & nacho[[input$tabs]] <= input$threshold[2], ]
               outliers_data <- nacho[nacho[[input$tabs]] <= input$threshold[1] | nacho[[input$tabs]] >= input$threshold[2], ]
-            } else if (input$tabs == "FoV" | input$tabs == "LoD" | input$tabs == "PC") {
+            } else if (input$tabs %in% c("FoV", "LoD", "PC")) {
               local_data <- nacho[nacho[[input$tabs]] >= input$threshold[1], ]
               outliers_data <- nacho[nacho[[input$tabs]] <= input$threshold[1], ]
             }
@@ -569,7 +644,6 @@ visualise <- function(nacho_object) {
               ) +
               ggplot2::labs(
                 x = input$attribute,
-                # y = paste(labels[input$tabs], units[input$tabs], sep = "\n"),
                 y = parse(text = paste0('paste("', labels[input$tabs], '", " ", ',  units[input$tabs], ")")),
                 colour = input$meta
               )
@@ -851,7 +925,7 @@ visualise <- function(nacho_object) {
             p
           },
           "norm" = {
-            shiny::req(input$tabs%in%c("pfbt", "hf", "norm_res"))
+            shiny::req(input$tabs %in% c("pfbt", "hf", "norm_res"))
             shiny::req(input$meta)
             colour_name <- input$meta
 
@@ -997,7 +1071,12 @@ visualise <- function(nacho_object) {
     }
   )
 
-  shiny::runApp(app)
+  if (!interactive()) {
+    message('[NACHO] Must be run in an interactive R session!')
+    return(app)
+  } else {
+    shiny::runApp(app)
+  }
   # nocov end
 }
 
